@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: getinmemory.c,v 1.8 2004/11/02 08:26:55 bagder Exp $
+ * $Id: getinmemory.c,v 1.11 2007-07-16 21:22:12 danf Exp $
  *
  * Example source code to show how the callback function can be used to
  * download data into a chunk of memory instead of storing it in a file.
@@ -14,6 +14,8 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include <curl/curl.h>
 #include <curl/types.h>
@@ -24,7 +26,7 @@ struct MemoryStruct {
   size_t size;
 };
 
-void *myrealloc(void *ptr, size_t size)
+static void *myrealloc(void *ptr, size_t size)
 {
   /* There might be a realloc() out there that doesn't like reallocing
      NULL pointers, so we take care of it here */
@@ -34,10 +36,10 @@ void *myrealloc(void *ptr, size_t size)
     return malloc(size);
 }
 
-size_t
+static size_t
 WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data)
 {
-  register int realsize = size * nmemb;
+  size_t realsize = size * nmemb;
   struct MemoryStruct *mem = (struct MemoryStruct *)data;
 
   mem->memory = (char *)myrealloc(mem->memory, mem->size + realsize + 1);
@@ -93,5 +95,8 @@ int main(int argc, char **argv)
    * you're done with it, you should free() it as a nice application.
    */
 
+  if(chunk.memory)
+    free(chunk.memory);
+
   return 0;
 }
