@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2004, 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 2004 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: strerror.c,v 1.33 2006-04-26 17:27:36 giva Exp $
+ * $Id: strerror.c,v 1.47 2007-08-31 19:36:33 danf Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -58,38 +58,35 @@ curl_easy_strerror(CURLcode error)
 #ifndef CURL_DISABLE_VERBOSE_STRINGS
   switch (error) {
   case CURLE_OK:
-    return "no error";
+    return "No error";
 
   case CURLE_UNSUPPORTED_PROTOCOL:
-    return "unsupported protocol";
+    return "Unsupported protocol";
 
   case CURLE_FAILED_INIT:
-    return "failed init";
+    return "Failed initialization";
 
   case CURLE_URL_MALFORMAT:
     return "URL using bad/illegal format or missing URL";
 
   case CURLE_COULDNT_RESOLVE_PROXY:
-    return "couldn't resolve proxy name";
+    return "Couldn't resolve proxy name";
 
   case CURLE_COULDNT_RESOLVE_HOST:
-    return "couldn't resolve host name";
+    return "Couldn't resolve host name";
 
   case CURLE_COULDNT_CONNECT:
-    return "couldn't connect to server";
+    return "Couldn't connect to server";
 
   case CURLE_FTP_WEIRD_SERVER_REPLY:
     return "FTP: weird server reply";
 
-  case CURLE_FTP_ACCESS_DENIED:
-    return "FTP: access denied";
+  case CURLE_REMOTE_ACCESS_DENIED:
+    return "Access denied to remote resource";
 
   case CURLE_FTP_WEIRD_PASS_REPLY:
     return "FTP: unknown PASS reply";
 
-  case CURLE_FTP_WEIRD_USER_REPLY:
-    return "FTP: unknown USER reply";
-
   case CURLE_FTP_WEIRD_PASV_REPLY:
     return "FTP: unknown PASV reply";
 
@@ -99,11 +96,8 @@ curl_easy_strerror(CURLcode error)
   case CURLE_FTP_CANT_GET_HOST:
     return "FTP: can't figure out the host in the PASV response";
 
-  case CURLE_FTP_CANT_RECONNECT:
-    return "FTP: can't connect to server the response code is unknown";
-
-  case CURLE_FTP_COULDNT_SET_BINARY:
-    return "FTP: couldn't set binary mode";
+  case CURLE_FTP_COULDNT_SET_TYPE:
+    return "FTP: couldn't set file type";
 
   case CURLE_PARTIAL_FILE:
     return "Transferred a partial file";
@@ -111,60 +105,47 @@ curl_easy_strerror(CURLcode error)
   case CURLE_FTP_COULDNT_RETR_FILE:
     return "FTP: couldn't retrieve (RETR failed) the specified file";
 
-  case CURLE_FTP_WRITE_ERROR:
-    return "FTP: the post-transfer acknowledge response was not OK";
-
-  case CURLE_FTP_QUOTE_ERROR:
-    return "FTP: a quote command returned error";
+  case CURLE_QUOTE_ERROR:
+    return "Quote command returned error";
 
   case CURLE_HTTP_RETURNED_ERROR:
     return "HTTP response code said error";
 
   case CURLE_WRITE_ERROR:
-    return "failed writing received data to disk/application";
+    return "Failed writing received data to disk/application";
 
-  case CURLE_FTP_COULDNT_STOR_FILE:
-    return "failed FTP upload (the STOR command)";
+  case CURLE_UPLOAD_FAILED:
+    return "Upload failed (at start/before it took off)";
 
   case CURLE_READ_ERROR:
-    return "failed to open/read local data from file/application";
+    return "Failed to open/read local data from file/application";
 
   case CURLE_OUT_OF_MEMORY:
-#ifdef CURL_DOES_CONVERSIONS
-    return "conversion failed -or- out of memory";
-#else
-    return "out of memory";
-#endif /* CURL_DOES_CONVERSIONS */
+    return "Out of memory";
 
-  case CURLE_OPERATION_TIMEOUTED:
-    return "a timeout was reached";
-
-  case CURLE_FTP_COULDNT_SET_ASCII:
-    return "FTP could not set ASCII mode (TYPE A)";
+  case CURLE_OPERATION_TIMEDOUT:
+    return "Timeout was reached";
 
   case CURLE_FTP_PORT_FAILED:
-    return "FTP command PORT failed";
+    return "FTP: command PORT failed";
 
   case CURLE_FTP_COULDNT_USE_REST:
-    return "FTP command REST failed";
+    return "FTP: command REST failed";
 
-  case CURLE_FTP_COULDNT_GET_SIZE:
-    return "FTP command SIZE failed";
-
-  case CURLE_HTTP_RANGE_ERROR:
-    return "a range was requested but the server did not deliver it";
+  case CURLE_RANGE_ERROR:
+    return "Requested range was not delivered by the server";
 
   case CURLE_HTTP_POST_ERROR:
-    return "internal problem setting up the POST";
+    return "Internal problem setting up the POST";
 
   case CURLE_SSL_CONNECT_ERROR:
     return "SSL connect error";
 
   case CURLE_BAD_DOWNLOAD_RESUME:
-    return "couldn't resume download";
+    return "Couldn't resume download";
 
   case CURLE_FILE_COULDNT_READ_FILE:
-    return "couldn't read a file:// file";
+    return "Couldn't read a file:// file";
 
   case CURLE_LDAP_CANNOT_BIND:
     return "LDAP: cannot bind";
@@ -172,62 +153,59 @@ curl_easy_strerror(CURLcode error)
   case CURLE_LDAP_SEARCH_FAILED:
     return "LDAP: search failed";
 
-  case CURLE_LIBRARY_NOT_FOUND:
-    return "a required shared library was not found";
-
   case CURLE_FUNCTION_NOT_FOUND:
-    return "a required function in the shared library was not found";
+    return "A required function in the library was not found";
 
   case CURLE_ABORTED_BY_CALLBACK:
-    return "the operation was aborted by an application callback";
+    return "Operation was aborted by an application callback";
 
   case CURLE_BAD_FUNCTION_ARGUMENT:
-    return "a libcurl function was given a bad argument";
+    return "A libcurl function was given a bad argument";
 
   case CURLE_INTERFACE_FAILED:
-    return "failed binding local connection end";
+    return "Failed binding local connection end";
 
   case CURLE_TOO_MANY_REDIRECTS :
-    return "number of redirects hit maximum amount";
+    return "Number of redirects hit maximum amount";
 
   case CURLE_UNKNOWN_TELNET_OPTION:
-    return "User specified an unknown option";
+    return "User specified an unknown telnet option";
 
   case CURLE_TELNET_OPTION_SYNTAX :
     return "Malformed telnet option";
 
   case CURLE_SSL_PEER_CERTIFICATE:
-    return "SSL peer certificate was not ok";
+    return "SSL peer certificate was not OK";
 
   case CURLE_GOT_NOTHING:
-    return "server returned nothing (no headers, no data)";
+    return "Server returned nothing (no headers, no data)";
 
   case CURLE_SSL_ENGINE_NOTFOUND:
     return "SSL crypto engine not found";
 
   case CURLE_SSL_ENGINE_SETFAILED:
-    return "can not set SSL crypto engine as default";
+    return "Can not set SSL crypto engine as default";
 
   case CURLE_SSL_ENGINE_INITFAILED:
-    return "failed to initialise SSL crypto engine";
+    return "Failed to initialise SSL crypto engine";
 
   case CURLE_SEND_ERROR:
-    return "failed sending data to the peer";
+    return "Failed sending data to the peer";
 
   case CURLE_RECV_ERROR:
-    return "failure when receiving data from the peer";
-
-  case CURLE_SHARE_IN_USE:
-    return "share is already in use";
+    return "Failure when receiving data from the peer";
 
   case CURLE_SSL_CERTPROBLEM:
-    return "problem with the local SSL certificate";
+    return "Problem with the local SSL certificate";
 
   case CURLE_SSL_CIPHER:
-    return "couldn't use specified SSL cipher";
+    return "Couldn't use specified SSL cipher";
 
   case CURLE_SSL_CACERT:
-    return "problem with the SSL CA cert (path? access rights?)";
+    return "Peer certificate cannot be authenticated with known CA certificates";
+
+  case CURLE_SSL_CACERT_BADFILE:
+    return "Problem with the SSL CA cert (path? access rights?)";
 
   case CURLE_BAD_CONTENT_ENCODING:
     return "Unrecognized HTTP Content-Encoding";
@@ -238,49 +216,65 @@ curl_easy_strerror(CURLcode error)
   case CURLE_FILESIZE_EXCEEDED:
     return "Maximum file size exceeded";
 
-  case CURLE_FTP_SSL_FAILED:
-    return "Requested FTP SSL level failed";
+  case CURLE_USE_SSL_FAILED:
+    return "Requested SSL level failed";
+
+  case CURLE_SSL_SHUTDOWN_FAILED:
+    return "Failed to shut down the SSL connection";
 
   case CURLE_SEND_FAIL_REWIND:
     return "Send failed since rewinding of the data stream failed";
 
   case CURLE_LOGIN_DENIED:
-    return "FTP: login denied";;
+    return "Login denied";
 
   case CURLE_TFTP_NOTFOUND:
-    return "TFTP: File Not Found";;
+    return "TFTP: File Not Found";
 
   case CURLE_TFTP_PERM:
-    return "TFTP: Access Violation";;
+    return "TFTP: Access Violation";
 
-  case CURLE_TFTP_DISKFULL:
-    return "TFTP: Disk full or allocation exceeded";;
+  case CURLE_REMOTE_DISK_FULL:
+    return "Disk full or allocation exceeded";
 
   case CURLE_TFTP_ILLEGAL:
-    return "TFTP: Illegal operation";;
+    return "TFTP: Illegal operation";
 
   case CURLE_TFTP_UNKNOWNID:
-    return "TFTP: Unknown transfer ID";;
+    return "TFTP: Unknown transfer ID";
 
-  case CURLE_TFTP_EXISTS:
-    return "TFTP: File already exists";;
+  case CURLE_REMOTE_FILE_EXISTS:
+    return "Remote file already exists";
 
   case CURLE_TFTP_NOSUCHUSER:
-    return "TFTP: No such user";;
+    return "TFTP: No such user";
 
   case CURLE_CONV_FAILED:
-    return "conversion failed";
+    return "Conversion failed";
 
   case CURLE_CONV_REQD:
-    return "caller must register CURLOPT_CONV_ callback options";
+    return "Caller must register CURLOPT_CONV_ callback options";
+
+  case CURLE_REMOTE_FILE_NOT_FOUND:
+    return "Remote file not found";
+
+  case CURLE_SSH:
+    return "Error in the SSH layer";
 
     /* error codes not used by current libcurl */
-  case CURLE_URL_MALFORMAT_USER:
-  case CURLE_FTP_USER_PASSWORD_INCORRECT:
-  case CURLE_MALFORMAT_USER:
-  case CURLE_BAD_CALLING_ORDER:
-  case CURLE_BAD_PASSWORD_ENTERED:
-  case CURLE_OBSOLETE:
+  case CURLE_OBSOLETE4:
+  case CURLE_OBSOLETE10:
+  case CURLE_OBSOLETE12:
+  case CURLE_OBSOLETE16:
+  case CURLE_OBSOLETE20:
+  case CURLE_OBSOLETE24:
+  case CURLE_OBSOLETE29:
+  case CURLE_OBSOLETE32:
+  case CURLE_OBSOLETE40:
+  case CURLE_OBSOLETE44:
+  case CURLE_OBSOLETE46:
+  case CURLE_OBSOLETE50:
+  case CURLE_OBSOLETE57:
   case CURL_LAST:
     break;
   }
@@ -298,12 +292,12 @@ curl_easy_strerror(CURLcode error)
    * The line number for the error will be near this comment, which
    * is why it is here, and not at the start of the switch.
    */
-  return "unknown error";
+  return "Unknown error";
 #else
   if (error == CURLE_OK)
-    return "no error";
+    return "No error";
   else
-    return "error";
+    return "Error";
 #endif
 }
 
@@ -313,39 +307,39 @@ curl_multi_strerror(CURLMcode error)
 #ifndef CURL_DISABLE_VERBOSE_STRINGS
   switch (error) {
   case CURLM_CALL_MULTI_PERFORM:
-    return "please call curl_multi_perform() soon";
+    return "Please call curl_multi_perform() soon";
 
   case CURLM_OK:
-    return "no error";
+    return "No error";
 
   case CURLM_BAD_HANDLE:
-    return "invalid multi handle";
+    return "Invalid multi handle";
 
   case CURLM_BAD_EASY_HANDLE:
-    return "invalid easy handle";
+    return "Invalid easy handle";
 
   case CURLM_OUT_OF_MEMORY:
-    return "out of memory";
+    return "Out of memory";
 
   case CURLM_INTERNAL_ERROR:
-    return "internal error";
+    return "Internal error";
 
   case CURLM_BAD_SOCKET:
-    return "invalid socket argument";
+    return "Invalid socket argument";
 
   case CURLM_UNKNOWN_OPTION:
-    return "unknown option";
+    return "Unknown option";
 
   case CURLM_LAST:
     break;
   }
 
-  return "unknown error";
+  return "Unknown error";
 #else
   if (error == CURLM_OK)
-    return "no error";
+    return "No error";
   else
-    return "error";
+    return "Error";
 #endif
 }
 
@@ -355,34 +349,34 @@ curl_share_strerror(CURLSHcode error)
 #ifndef CURL_DISABLE_VERBOSE_STRINGS
   switch (error) {
   case CURLSHE_OK:
-    return "no error";
+    return "No error";
 
   case CURLSHE_BAD_OPTION:
-    return "unknown share option";
+    return "Unknown share option";
 
   case CURLSHE_IN_USE:
-    return "share currently in use";
+    return "Share currently in use";
 
   case CURLSHE_INVALID:
-    return "invalid share handle";
+    return "Invalid share handle";
 
   case CURLSHE_NOMEM:
-    return "out of memory";
+    return "Out of memory";
 
   case CURLSHE_LAST:
     break;
   }
 
-  return "CURLSH unknown";
+  return "CURLSHcode unknown";
 #else
   if (error == CURLSHE_OK)
-    return "no error";
+    return "No error";
   else
-    return "error";
+    return "Error";
 #endif
 }
 
-#if defined(WIN32) && !defined(__CYGWIN__)
+#ifdef USE_WINSOCK
 
 /* This function handles most / all (?) Winsock errors cURL is able to produce.
  */
@@ -394,7 +388,7 @@ get_winsock_error (int err, char *buf, size_t len)
 #ifndef CURL_DISABLE_VERBOSE_STRINGS
   switch (err) {
   case WSAEINTR:
-    p = "Call interrupted.";
+    p = "Call interrupted";
     break;
   case WSAEBADF:
     p = "Bad file";
@@ -419,7 +413,7 @@ get_winsock_error (int err, char *buf, size_t len)
     p = "Blocking call in progress";
     break;
   case WSAENOTSOCK:
-    p = "Descriptor is not a socket.";
+    p = "Descriptor is not a socket";
     break;
   case WSAEDESTADDRREQ:
     p = "Need destination address";
@@ -533,7 +527,7 @@ get_winsock_error (int err, char *buf, size_t len)
     p = "Winsock library not initialised";
     break;
   case WSAVERNOTSUPPORTED:
-    p = "Winsock version not supported.";
+    p = "Winsock version not supported";
     break;
 
     /* getXbyY() errors (already handled in herrmsg):
@@ -561,7 +555,7 @@ get_winsock_error (int err, char *buf, size_t len)
     return NULL;
   }
 #else
-  if (error == CURLE_OK)
+  if (err == CURLE_OK)
     return NULL;
   else
     p = "error";
@@ -570,14 +564,14 @@ get_winsock_error (int err, char *buf, size_t len)
   buf [len-1] = '\0';
   return buf;
 }
-#endif   /* WIN32 && !__CYGWIN__ */
+#endif   /* USE_WINSOCK */
 
 /*
  * Our thread-safe and smart strerror() replacement.
  *
  * The 'err' argument passed in to this function MUST be a true errno number
  * as reported on this system. We do no range checking on the number before
- * we pass it to the "number-to-message" convertion function and there might
+ * we pass it to the "number-to-message" conversion function and there might
  * be systems that don't do proper range checking in there themselves.
  *
  * We don't do range checking (on systems other than Windows) since there is
@@ -588,14 +582,14 @@ const char *Curl_strerror(struct connectdata *conn, int err)
   char *buf, *p;
   size_t max;
 
-  curlassert(conn);
-  curlassert(err >= 0);
+  DEBUGASSERT(conn);
+  DEBUGASSERT(err >= 0);
 
   buf = conn->syserr_buf;
   max = sizeof(conn->syserr_buf)-1;
   *buf = '\0';
 
-#if defined(WIN32) && !defined(__CYGWIN__)
+#ifdef USE_WINSOCK
 
 #ifdef _WIN32_WCE
   buf[0]=0;
@@ -619,7 +613,7 @@ const char *Curl_strerror(struct connectdata *conn, int err)
       snprintf(buf, max, "Unknown error %d (%#x)", err, err);
   }
 #endif
-#else /* not native Windows coming up */
+#else /* not USE_WINSOCK coming up */
 
   /* These should be atomic and hopefully thread-safe */
 #ifdef HAVE_STRERROR_R
@@ -637,14 +631,17 @@ const char *Curl_strerror(struct connectdata *conn, int err)
     char *msg = strerror_r(err, buffer, sizeof(buffer));
     /* this version of strerror_r() only *might* use the buffer we pass to
        the function, but it always returns the error message as a pointer,
-       so we must copy that string unconditionally */
-    strncpy(buf, msg, max);
+       so we must copy that string unconditionally (if non-NULL) */
+    if(msg)
+      strncpy(buf, msg, max);
+    else
+      snprintf(buf, max, "Unknown error %d", err);
   }
 #endif /* end of HAVE_GLIBC_STRERROR_R */
 #else /* HAVE_STRERROR_R */
   strncpy(buf, strerror(err), max);
 #endif /* end of HAVE_STRERROR_R */
-#endif /* end of ! Windows */
+#endif /* end of ! USE_WINSOCK */
 
   buf[max] = '\0'; /* make sure the string is zero terminated */
 
@@ -670,7 +667,7 @@ const char *Curl_idn_strerror (struct connectdata *conn, int err)
   char *buf;
   size_t max;
 
-  curlassert(conn);
+  DEBUGASSERT(conn);
 
   buf = conn->syserr_buf;
   max = sizeof(conn->syserr_buf)-1;
@@ -699,7 +696,7 @@ const char *Curl_idn_strerror (struct connectdata *conn, int err)
       str = "No ACE prefix (\"xn--\")";
       break;
     case IDNA_ROUNDTRIP_VERIFY_ERROR:
-      str = "Roundtrip verify error";
+      str = "Round trip verify error";
       break;
     case IDNA_CONTAINS_ACE_PREFIX:
       str = "Already have ACE prefix (\"xn--\")";
@@ -722,7 +719,7 @@ const char *Curl_idn_strerror (struct connectdata *conn, int err)
   if ((Idna_rc)err == IDNA_SUCCESS)
     str = "No error";
   else
-    str = "error";
+    str = "Error";
 #endif
   if (str)
     strncpy(buf, str, max);
