@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: tftp.c,v 1.26 2006-05-09 13:02:53 bagder Exp $
+ * $Id: tftp.c,v 1.51 2007-09-07 20:35:37 danf Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -30,12 +30,6 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <ctype.h>
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
 
 #if defined(WIN32)
 #include <time.h>
@@ -45,7 +39,9 @@
 #include <sys/socket.h>
 #endif
 #include <netinet/in.h>
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -63,8 +59,7 @@
 #include <sys/param.h>
 #endif
 
-
-#endif
+#endif /* WIN32 */
 
 #include "urldata.h"
 #include <curl/curl.h>
@@ -75,6 +70,7 @@
 #include "connect.h"
 #include "strerror.h"
 #include "sockaddr.h" /* required for Curl_sockaddr_storage */
+#include "url.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
@@ -118,7 +114,10 @@ typedef enum {
   TFTP_ERR_ILLEGAL,
   TFTP_ERR_UNKNOWNID,
   TFTP_ERR_EXISTS,
-  TFTP_ERR_NOSUCHUSER,
+  TFTP_ERR_NOSUCHUSER,	/* This will never be triggered by this code */
+
+  /* The remaining error codes are internal to curl */
+  TFTP_ERR_NONE = -100,
   TFTP_ERR_TIMEOUT,
   TFTP_ERR_NORESPONSE
 } tftp_error_t;
@@ -140,7 +139,6 @@ typedef struct tftp_state_data {
   time_t          max_time;
   unsigned short  block;
   struct Curl_sockaddr_storage   local_addr;
-  socklen_t       local_addrlen;
   struct Curl_sockaddr_storage   remote_addr;
   socklen_t       remote_addrlen;
   int             rbytes;
@@ -153,7 +151,6 @@ typedef struct tftp_state_data {
 /* Forward declarations */
 static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event) ;
 static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event) ;
-void tftp_set_timeouts(tftp_state_data_t *state) ;
 
 /**********************************************************
  *
@@ -165,7 +162,7 @@ void tftp_set_timeouts(tftp_state_data_t *state) ;
  *
  *
  **********************************************************/
-void tftp_set_timeouts(tftp_state_data_t *state)
+static void tftp_set_timeouts(tftp_state_data_t *state)
 {
 
   struct SessionHandle *data = state->conn->data;
@@ -174,7 +171,8 @@ void tftp_set_timeouts(tftp_state_data_t *state)
   time(&state->start_time);
   if(state->state == TFTP_STATE_START) {
     /* Compute drop-dead time */
-    maxtime = (time_t)(data->set.connecttimeout?data->set.connecttimeout:30);
+    maxtime = (time_t)(data->set.connecttimeout/1000L?
+                       data->set.connecttimeout/1000L:30);
     state->max_time = state->start_time+maxtime;
 
     /* Set per-block timeout to total */
@@ -183,6 +181,10 @@ void tftp_set_timeouts(tftp_state_data_t *state)
     /* Average restart after 5 seconds */
     state->retry_max = timeout/5;
 
+    if(state->retry_max < 1)
+      /* avoid division by zero below */
+      state->retry_max = 1;
+
     /* Compute the re-start interval to suit the timeout */
     state->retry_time = timeout/state->retry_max;
     if(state->retry_time<1)
@@ -192,7 +194,8 @@ void tftp_set_timeouts(tftp_state_data_t *state)
   else {
 
     /* Compute drop-dead time */
-    maxtime = (time_t)(data->set.timeout?data->set.timeout:3600);
+    maxtime = (time_t)(data->set.timeout/1000L?
+                       data->set.timeout/1000L:3600);
     state->max_time = state->start_time+maxtime;
 
     /* Set per-block timeout to 10% of total */
@@ -228,40 +231,37 @@ void tftp_set_timeouts(tftp_state_data_t *state)
 
 static void setpacketevent(tftp_packet_t *packet, unsigned short num)
 {
-  packet->data[0] = (num >> 8);
-  packet->data[1] = (num & 0xff);
+  packet->data[0] = (unsigned char)(num >> 8);
+  packet->data[1] = (unsigned char)(num & 0xff);
 }
 
 
 static void setpacketblock(tftp_packet_t *packet, unsigned short num)
 {
-  packet->data[2] = (num >> 8);
-  packet->data[3] = (num & 0xff);
+  packet->data[2] = (unsigned char)(num >> 8);
+  packet->data[3] = (unsigned char)(num & 0xff);
 }
 
-static unsigned short getrpacketevent(tftp_packet_t *packet)
+static unsigned short getrpacketevent(const tftp_packet_t *packet)
 {
-  return (packet->data[0] << 8) | packet->data[1];
+  return (unsigned short)((packet->data[0] << 8) | packet->data[1]);
 }
 
-static unsigned short getrpacketblock(tftp_packet_t *packet)
+static unsigned short getrpacketblock(const tftp_packet_t *packet)
 {
-  return (packet->data[2] << 8) | packet->data[3];
+  return (unsigned short)((packet->data[2] << 8) | packet->data[3]);
 }
 
 static CURLcode tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
 {
   int sbytes;
   const char *mode = "octet";
-
-  /* As RFC3617 describes the separator slash is not actually part of the file
-     name so we skip the always-present first letter of the path string. */
-  char *filename = &state->conn->path[1];
+  char *filename;
   struct SessionHandle *data = state->conn->data;
   CURLcode res = CURLE_OK;
 
   /* Set ascii mode if -B flag was used */
-  if(data->set.ftp_ascii)
+  if(data->set.prefer_ascii)
     mode = "netascii";
 
   switch(event) {
@@ -279,8 +279,7 @@ static CURLcode tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
     if(data->set.upload) {
       /* If we are uploading, send an WRQ */
       setpacketevent(&state->spacket, TFTP_EVENT_WRQ);
-      filename = curl_easy_unescape(data, filename, 0, NULL);
-      state->conn->upload_fromhere = (char *)&state->spacket.data[4];
+      state->conn->data->reqdata.upload_fromhere = (char *)&state->spacket.data[4];
       if(data->set.infilesize != -1)
         Curl_pgrsSetUploadSize(data, data->set.infilesize);
     }
@@ -288,6 +287,13 @@ static CURLcode tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
       /* If we are downloading, send an RRQ */
       setpacketevent(&state->spacket, TFTP_EVENT_RRQ);
     }
+    /* As RFC3617 describes the separator slash is not actually part of the
+    file name so we skip the always-present first letter of the path string. */
+    filename = curl_easy_unescape(data, &state->conn->data->reqdata.path[1], 0,
+                                  NULL);
+    if (!filename)
+      return CURLE_OUT_OF_MEMORY;
+
     snprintf((char *)&state->spacket.data[2],
              TFTP_BLOCKSIZE,
              "%s%c%s%c", filename, '\0',  mode, '\0');
@@ -297,8 +303,9 @@ static CURLcode tftp_send_first(tftp_state_data_t *state, tftp_event_t event)
                     state->conn->ip_addr->ai_addr,
                     state->conn->ip_addr->ai_addrlen);
     if(sbytes < 0) {
-      failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
+      failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
     }
+    Curl_safefree(filename);
     break;
 
   case TFTP_EVENT_ACK: /* Connected for transmit */
@@ -355,7 +362,7 @@ static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event)
       }
     }
     /* This is the expected block.  Reset counters and ACK it. */
-    state->block = rblock;
+    state->block = (unsigned short)rblock;
     state->retries = 0;
     setpacketevent(&state->spacket, TFTP_EVENT_ACK);
     setpacketblock(&state->spacket, state->block);
@@ -364,7 +371,8 @@ static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event)
                     (struct sockaddr *)&state->remote_addr,
                     state->remote_addrlen);
     if(sbytes < 0) {
-      failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
+      failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
+      return CURLE_SEND_ERROR;
     }
 
     /* Check if completed (That is, a less than full packet is received) */
@@ -393,7 +401,8 @@ static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event)
                       state->remote_addrlen);
       /* Check all sbytes were sent */
       if(sbytes<0) {
-        failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
+        failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
+        return CURLE_SEND_ERROR;
       }
     }
     break;
@@ -404,10 +413,9 @@ static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event)
 
   default:
     failf(data, "%s\n", "tftp_rx: internal error");
-    break;
+    return CURLE_TFTP_ILLEGAL; /* not really the perfect return code for
+                                  this */
   }
-  Curl_pgrsSetDownloadCounter(data,
-                              (curl_off_t) state->block*TFTP_BLOCKSIZE);
   return CURLE_OK;
 }
 
@@ -424,6 +432,7 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
   int sbytes;
   int rblock;
   CURLcode res = CURLE_OK;
+  struct Curl_transfer_keeper *k = &data->reqdata.keep;
 
   switch(event) {
 
@@ -450,7 +459,7 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
                         state->remote_addrlen);
         /* Check all sbytes were sent */
         if(sbytes<0) {
-          failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
+          failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
           res = CURLE_SEND_ERROR;
         }
       }
@@ -475,8 +484,12 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
                     state->remote_addrlen);
     /* Check all sbytes were sent */
     if(sbytes<0) {
-      failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
+      failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
+      return CURLE_SEND_ERROR;
     }
+    /* Update the progress meter */
+    k->writebytecount += state->sbytes;
+    Curl_pgrsSetUploadCounter(data, k->writebytecount);
     break;
 
   case TFTP_EVENT_TIMEOUT:
@@ -488,7 +501,8 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
     if(state->retries > state->retry_max) {
       state->error = TFTP_ERR_TIMEOUT;
       state->state = TFTP_STATE_FIN;
-    } else {
+    }
+    else {
       /* Re-send the data packet */
       sbytes = sendto(state->sockfd, (void *)&state->spacket,
                       4+state->sbytes, SEND_4TH_ARG,
@@ -496,8 +510,11 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
                       state->remote_addrlen);
       /* Check all sbytes were sent */
       if(sbytes<0) {
-        failf(data, "%s\n", Curl_strerror(state->conn, Curl_sockerrno()));
+        failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
+        return CURLE_SEND_ERROR;
       }
+      /* since this was a re-send, we remain at the still byte position */
+      Curl_pgrsSetUploadCounter(data, k->writebytecount);
     }
     break;
 
@@ -510,9 +527,6 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
     break;
   }
 
-  /* Update the progress meter */
-  Curl_pgrsSetUploadCounter(data, (curl_off_t) state->block*TFTP_BLOCKSIZE);
-
   return res;
 }
 
@@ -567,42 +581,44 @@ CURLcode Curl_tftp_connect(struct connectdata *conn, bool *done)
   tftp_state_data_t     *state;
   int rc;
 
-  state = conn->proto.tftp = calloc(sizeof(tftp_state_data_t), 1);
+  state = conn->data->reqdata.proto.tftp = calloc(sizeof(tftp_state_data_t),
+                                                  1);
   if(!state)
     return CURLE_OUT_OF_MEMORY;
 
+  conn->bits.close = FALSE; /* keep it open if possible */
+
   state->conn = conn;
   state->sockfd = state->conn->sock[FIRSTSOCKET];
   state->state = TFTP_STATE_START;
+  state->error = TFTP_ERR_NONE;
 
-#ifdef WIN32
-  /* AF_UNSPEC == 0 (from above calloc) doesn't work on Winsock */
-
-  /* NOTE: this blatantly assumes IPv4. This should be fixed! */
-  ((struct sockaddr_in*)&state->local_addr)->sin_family =
-    conn->ip_addr->ai_family;
-#endif
+  ((struct sockaddr *)&state->local_addr)->sa_family =
+    (unsigned short)(conn->ip_addr->ai_family);
 
   tftp_set_timeouts(state);
 
-  /* Bind to any interface, random UDP port.
-   *
-   * We once used the size of the local_addr struct as the third argument for
-   * bind() to better work with IPv6 or whatever size the struct could have,
-   * but we learned that at least Tru64, AIX and IRIX *requires* the size of
-   * that argument to match the exact size of a 'sockaddr_in' struct when
-   * running IPv4-only.
-   *
-   * Therefore we use the size from the address we connected to, which we
-   * assume uses the same IP version and thus hopefully this works for both
-   * IPv4 and IPv6...
-   */
-  rc = bind(state->sockfd, (struct sockaddr *)&state->local_addr,
-            conn->ip_addr->ai_addrlen);
-  if(rc) {
-    failf(conn->data, "bind() failed; %s\n",
-          Curl_strerror(conn, Curl_sockerrno()));
-    return CURLE_COULDNT_CONNECT;
+  if(!conn->bits.reuse) {
+    /* If not reused, bind to any interface, random UDP port. If it is reused,
+     * this has already been done!
+     *
+     * We once used the size of the local_addr struct as the third argument for
+     * bind() to better work with IPv6 or whatever size the struct could have,
+     * but we learned that at least Tru64, AIX and IRIX *requires* the size of
+     * that argument to match the exact size of a 'sockaddr_in' struct when
+     * running IPv4-only.
+     *
+     * Therefore we use the size from the address we connected to, which we
+     * assume uses the same IP version and thus hopefully this works for both
+     * IPv4 and IPv6...
+     */
+    rc = bind(state->sockfd, (struct sockaddr *)&state->local_addr,
+              conn->ip_addr->ai_addrlen);
+    if(rc) {
+      failf(conn->data, "bind() failed; %s\n",
+            Curl_strerror(conn, SOCKERRNO));
+      return CURLE_COULDNT_CONNECT;
+    }
   }
 
   Curl_pgrsStartNow(conn->data);
@@ -619,12 +635,16 @@ CURLcode Curl_tftp_connect(struct connectdata *conn, bool *done)
  * The done callback
  *
  **********************************************************/
-CURLcode Curl_tftp_done(struct connectdata *conn, CURLcode status)
+CURLcode Curl_tftp_done(struct connectdata *conn, CURLcode status,
+                        bool premature)
 {
   (void)status; /* unused */
+  (void)premature; /* not used */
 
-  free(conn->proto.tftp);
-  conn->proto.tftp = NULL;
+#if 0
+  free(conn->data->reqdata.proto.tftp);
+  conn->data->reqdata.proto.tftp = NULL;
+#endif
   Curl_pgrsDone(conn);
 
   return CURLE_OK;
@@ -644,27 +664,47 @@ CURLcode Curl_tftp_done(struct connectdata *conn, CURLcode status)
 CURLcode Curl_tftp(struct connectdata *conn, bool *done)
 {
   struct SessionHandle  *data = conn->data;
-  tftp_state_data_t     *state = (tftp_state_data_t *)(conn->proto.tftp);
+  tftp_state_data_t     *state =
+    (tftp_state_data_t *) conn->data->reqdata.proto.tftp;
   tftp_event_t          event;
   CURLcode              code;
   int                   rc;
   struct Curl_sockaddr_storage fromaddr;
   socklen_t             fromlen;
   int                   check_time = 0;
+  struct Curl_transfer_keeper *k = &data->reqdata.keep;
 
-  (void)done; /* prevent compiler warning */
+  *done = TRUE;
+
+  /*
+    Since connections can be re-used between SessionHandles, this might be a
+    connection already existing but on a fresh SessionHandle struct so we must
+    make sure we have a good 'struct TFTP' to play with. For new connections,
+    the struct TFTP is allocated and setup in the Curl_tftp_connect() function.
+  */
+  if(!state) {
+    code = Curl_tftp_connect(conn, done);
+    if(code)
+      return code;
+    state = (tftp_state_data_t *)conn->data->reqdata.proto.tftp;
+  }
+
+  code = Curl_readwrite_init(conn);
+  if(code)
+    return code;
 
   /* Run the TFTP State Machine */
-  for(tftp_state_machine(state, TFTP_EVENT_INIT);
-      state->state != TFTP_STATE_FIN;
-      tftp_state_machine(state, event) ) {
+  for(code=tftp_state_machine(state, TFTP_EVENT_INIT);
+      (state->state != TFTP_STATE_FIN) && (code == CURLE_OK);
+      code=tftp_state_machine(state, event) ) {
 
     /* Wait until ready to read or timeout occurs */
-    rc=Curl_select(state->sockfd, CURL_SOCKET_BAD, state->retry_time * 1000);
+    rc=Curl_socket_ready(state->sockfd, CURL_SOCKET_BAD,
+                         state->retry_time * 1000);
 
     if(rc == -1) {
       /* bail out */
-      int error = Curl_sockerrno();
+      int error = SOCKERRNO;
       failf(data, "%s\n", Curl_strerror(conn, error));
       event = TFTP_EVENT_ERROR;
     }
@@ -704,11 +744,13 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
           /* Don't pass to the client empty or retransmitted packets */
           if (state->rbytes > 4 &&
               ((state->block+1) == getrpacketblock(&state->rpacket))) {
-            code = Curl_client_write(data, CLIENTWRITE_BODY,
+            code = Curl_client_write(conn, CLIENTWRITE_BODY,
                                      (char *)&state->rpacket.data[4],
                                      state->rbytes-4);
             if(code)
               return code;
+            k->bytecount += state->rbytes-4;
+            Curl_pgrsSetDownloadCounter(data, (curl_off_t) k->bytecount);
           }
           break;
         case TFTP_EVENT_ERROR:
@@ -743,14 +785,16 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
     }
 
   }
+  if(code)
+    return code;
 
   /* Tell curl we're done */
-  code = Curl_Transfer(conn, -1, -1, FALSE, NULL, -1, NULL);
+  code = Curl_setup_transfer(conn, -1, -1, FALSE, NULL, -1, NULL);
   if(code)
     return code;
 
   /* If we have encountered an error */
-  if(state->error) {
+  if(state->error != TFTP_ERR_NONE) {
 
     /* Translate internal error codes to curl error codes */
     switch(state->error) {
@@ -761,8 +805,9 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
       code = CURLE_TFTP_PERM;
       break;
     case TFTP_ERR_DISKFULL:
-      code = CURLE_TFTP_DISKFULL;
+      code = CURLE_REMOTE_DISK_FULL;
       break;
+    case TFTP_ERR_UNDEF:
     case TFTP_ERR_ILLEGAL:
       code = CURLE_TFTP_ILLEGAL;
       break;
@@ -770,13 +815,13 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
       code = CURLE_TFTP_UNKNOWNID;
       break;
     case TFTP_ERR_EXISTS:
-      code = CURLE_TFTP_EXISTS;
+      code = CURLE_REMOTE_FILE_EXISTS;
       break;
     case TFTP_ERR_NOSUCHUSER:
       code = CURLE_TFTP_NOSUCHUSER;
       break;
     case TFTP_ERR_TIMEOUT:
-      code = CURLE_OPERATION_TIMEOUTED;
+      code = CURLE_OPERATION_TIMEDOUT;
       break;
     case TFTP_ERR_NORESPONSE:
       code = CURLE_COULDNT_CONNECT;
