@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: first.c,v 1.14 2006-10-25 09:20:44 yangtse Exp $
+ * $Id: first.c,v 1.16 2007-07-15 20:59:43 bagder Exp $
  */
 
 #include "test.h"
@@ -35,6 +35,10 @@ int select_test (int num_fds, fd_set *rd, fd_set *wr, fd_set *exc,
 }
 
 char *arg2=NULL;
+char *arg3=NULL;
+int test_argc;
+char **test_argv;
+
 
 int main(int argc, char **argv)
 {
@@ -64,9 +68,16 @@ int main(int argc, char **argv)
     fprintf(stderr, "Pass URL as argument please\n");
     return 1;
   }
+
+  test_argc = argc;
+  test_argv = argv;
+
   if(argc>2)
     arg2=argv[2];
 
+  if(argc>3)
+    arg3=argv[3];
+
   URL = argv[1]; /* provide this to the rest */
 
   fprintf(stderr, "URL: %s\n", URL);
