@@ -1,19 +1,24 @@
+/*****************************************************************************
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
+ *                             \___|\___/|_| \_\_____|
+ *
+ * $Id: lib506.c,v 1.15 2007-07-23 18:51:22 danf Exp $
+ */
+
 #include "test.h"
 #include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
 
+#include <mprintf.h>
+
 const char *HOSTHEADER = "Host: www.host.foo.com";
 const char *JAR = "log/jar506";
 #define THREADS 2
 
-void lock(CURL *handle, curl_lock_data data, curl_lock_access access,
-          void *useptr );
-void unlock(CURL *handle, curl_lock_data data, void *useptr );
-struct curl_slist *sethost(struct curl_slist *headers);
-void *fire(void *ptr);
-char *suburl(char *base, int i);
-
 /* struct containing data of a thread */
 struct Tdata {
   CURLSH *share;
@@ -26,7 +31,7 @@ struct userdata {
 };
 
 /* lock callback */
-void lock(CURL *handle, curl_lock_data data, curl_lock_access access,
+static void my_lock(CURL *handle, curl_lock_data data, curl_lock_access access,
           void *useptr )
 {
   const char *what;
@@ -37,13 +42,13 @@ void lock(CURL *handle, curl_lock_data data, curl_lock_access access,
 
   switch ( data ) {
     case CURL_LOCK_DATA_SHARE:
-      what = "share";  
+      what = "share";
       break;
     case CURL_LOCK_DATA_DNS:
-      what = "dns";  
+      what = "dns";
       break;
     case CURL_LOCK_DATA_COOKIE:
-      what = "cookie";  
+      what = "cookie";
       break;
     default:
       fprintf(stderr, "lock: no such data: %d\n", (int)data);
@@ -54,20 +59,20 @@ void lock(CURL *handle, curl_lock_data data, curl_lock_access access,
 }
 
 /* unlock callback */
-void unlock(CURL *handle, curl_lock_data data, void *useptr )
+static void my_unlock(CURL *handle, curl_lock_data data, void *useptr )
 {
   const char *what;
   struct userdata *user = (struct userdata *)useptr;
   (void)handle;
   switch ( data ) {
     case CURL_LOCK_DATA_SHARE:
-      what = "share";  
+      what = "share";
       break;
     case CURL_LOCK_DATA_DNS:
-      what = "dns";  
+      what = "dns";
       break;
     case CURL_LOCK_DATA_COOKIE:
-      what = "cookie";  
+      what = "cookie";
       break;
     default:
       fprintf(stderr, "unlock: no such data: %d\n", (int)data);
@@ -79,7 +84,7 @@ void unlock(CURL *handle, curl_lock_data data, void *useptr )
 
 
 /* build host entry */
-struct curl_slist *sethost(struct curl_slist *headers)
+static struct curl_slist *sethost(struct curl_slist *headers)
 {
   (void)headers;
   return curl_slist_append(NULL, HOSTHEADER );
@@ -87,14 +92,19 @@ struct curl_slist *sethost(struct curl_slist *headers)
 
 
 /* the dummy thread function */
-void *fire(void *ptr)
+static void *fire(void *ptr)
 {
   CURLcode code;
   struct curl_slist *headers;
   struct Tdata *tdata = (struct Tdata*)ptr;
-  CURL *curl = curl_easy_init();
+  CURL *curl;
   int i=0;
 
+  if ((curl = curl_easy_init()) == NULL) {
+    fprintf(stderr, "curl_easy_init() failed\n");
+    return NULL;
+  }
+
   headers = sethost(NULL);
   curl_easy_setopt(curl, CURLOPT_VERBOSE,    1);
   curl_easy_setopt(curl, CURLOPT_HTTPHEADER, (void*)headers);
@@ -118,17 +128,9 @@ void *fire(void *ptr)
 
 
 /* build request url */
-char *suburl(char *base, int i)
+static char *suburl(const char *base, int i)
 {
-  size_t len = strlen(base);
-  char *url = (char *)malloc(len+5);
-  if (!url) {
-    abort();
-  }
-  strcpy(url, base);
-  strcat(url, "0000");
-  url[len+3] = 48+i;
-  return url;
+  return curl_maprintf("%s000%c", base, 48+i);
 }
 
 
@@ -136,7 +138,7 @@ char *suburl(char *base, int i)
 int test(char *URL)
 {
   int res;
-  CURLSHcode scode;
+  CURLSHcode scode = CURLSHE_OK;
   char *url;
   struct Tdata tdata;
   CURL *curl;
@@ -147,42 +149,76 @@ int test(char *URL)
 
   user.text = (char *)"Pigs in space";
   user.counter = 0;
-  
+
   printf( "GLOBAL_INIT\n" );
-  curl_global_init( CURL_GLOBAL_ALL );
+  if (curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
+    fprintf(stderr, "curl_global_init() failed\n");
+    return TEST_ERR_MAJOR_BAD;
+  }
 
   /* prepare share */
   printf( "SHARE_INIT\n" );
-  share = curl_share_init();
-  curl_share_setopt( share, CURLSHOPT_LOCKFUNC,   lock);
-  curl_share_setopt( share, CURLSHOPT_UNLOCKFUNC, unlock);
-  curl_share_setopt( share, CURLSHOPT_USERDATA,   &user);
-  printf( "CURL_LOCK_DATA_COOKIE\n" );
-  curl_share_setopt( share, CURLSHOPT_SHARE,      CURL_LOCK_DATA_COOKIE);
-  printf( "CURL_LOCK_DATA_DNS\n" );
-  curl_share_setopt( share, CURLSHOPT_SHARE,      CURL_LOCK_DATA_DNS);
-  
+  if ((share = curl_share_init()) == NULL) {
+    fprintf(stderr, "curl_share_init() failed\n");
+    curl_global_cleanup();
+    return TEST_ERR_MAJOR_BAD;
+  }
+
+  if ( CURLSHE_OK == scode ) {
+    printf( "CURLSHOPT_LOCKFUNC\n" );
+    scode = curl_share_setopt( share, CURLSHOPT_LOCKFUNC, my_lock);
+  }
+  if ( CURLSHE_OK == scode ) {
+    printf( "CURLSHOPT_UNLOCKFUNC\n" );
+    scode = curl_share_setopt( share, CURLSHOPT_UNLOCKFUNC, my_unlock);
+  }
+  if ( CURLSHE_OK == scode ) {
+    printf( "CURLSHOPT_USERDATA\n" );
+    scode = curl_share_setopt( share, CURLSHOPT_USERDATA, &user);
+  }
+  if ( CURLSHE_OK == scode ) {
+    printf( "CURL_LOCK_DATA_COOKIE\n" );
+    scode = curl_share_setopt( share, CURLSHOPT_SHARE, CURL_LOCK_DATA_COOKIE);
+  }
+  if ( CURLSHE_OK == scode ) {
+    printf( "CURL_LOCK_DATA_DNS\n" );
+    scode = curl_share_setopt( share, CURLSHOPT_SHARE, CURL_LOCK_DATA_DNS);
+  }
+
+  if ( CURLSHE_OK != scode ) {
+    fprintf(stderr, "curl_share_setopt() failed\n");
+    curl_share_cleanup(share);
+    curl_global_cleanup();
+    return TEST_ERR_MAJOR_BAD;
+  }
+
+
   res = 0;
 
   /* start treads */
   for (i=1; i<=THREADS; i++ ) {
-    
+
     /* set thread data */
-    tdata.url   = suburl( URL, i ); /* must be freed */
+    tdata.url   = suburl( URL, i ); /* must be curl_free()d */
     tdata.share = share;
 
     /* simulate thread, direct call of "thread" function */
     printf( "*** run %d\n",i );
     fire( &tdata );
 
-    free( tdata.url );
+    curl_free( tdata.url );
 
   }
 
 
   /* fetch a another one and save cookies */
   printf( "*** run %d\n", i );
-  curl = curl_easy_init();
+  if ((curl = curl_easy_init()) == NULL) {
+    fprintf(stderr, "curl_easy_init() failed\n");
+    curl_share_cleanup(share);
+    curl_global_cleanup();
+    return TEST_ERR_MAJOR_BAD;
+  }
 
   url = suburl( URL, i );
   headers = sethost( NULL );
@@ -211,19 +247,19 @@ int test(char *URL)
   printf( "CLEANUP\n" );
   curl_easy_cleanup( curl );
   curl_slist_free_all( headers );
-  free(url);
-  
-  
+
+  curl_free(url);
+
   /* free share */
   printf( "SHARE_CLEANUP\n" );
   scode = curl_share_cleanup( share );
   if ( scode!=CURLSHE_OK )
     fprintf(stderr, "curl_share_cleanup failed, code errno %d\n",
             (int)scode);
-  
+
   printf( "GLOBAL_CLEANUP\n" );
   curl_global_cleanup();
- 
+
   return res;
 }
 
