@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sockfilt.c,v 1.26 2006-10-27 03:47:58 yangtse Exp $
+ * $Id: sockfilt.c,v 1.35 2007-08-30 18:26:19 danf Exp $
  ***************************************************************************/
 
 /* Purpose
@@ -175,8 +175,11 @@ static int juggle(curl_socket_t *sockfdp,
   ssize_t nread_socket;
   ssize_t bytes_written;
   ssize_t buffer_len;
-  unsigned char buffer[256]; /* FIX: bigger buffer */
-  char data[256];
+
+ /* 'buffer' is this excessively large only to be able to support things like
+    test 1003 which tests exceedingly large server response lines */
+  unsigned char buffer[17010];
+  char data[16];
 
   timeout.tv_sec = 120;
   timeout.tv_usec = 0;
@@ -238,7 +241,7 @@ static int juggle(curl_socket_t *sockfdp,
 
   do {
     rc = select(maxfd + 1, &fds_read, &fds_write, &fds_err, &timeout);
-  } while((rc == -1) && (our_sockerrno() == EINTR));
+  } while((rc == -1) && (SOCKERRNO == EINTR));
 
   switch(rc) {
   case -1:
@@ -280,7 +283,7 @@ static int juggle(curl_socket_t *sockfdp,
            Replies to PORT with "IPv[num]/[port]" */
         sprintf((char *)buffer, "IPv%d/%d\n", use_ipv6?6:4, (int)port);
         buffer_len = (ssize_t)strlen((char *)buffer);
-        sprintf(data, "PORT\n%04x\n", buffer_len);
+        snprintf(data, sizeof(data), "PORT\n%04x\n", buffer_len);
         write(fileno(stdout), data, 10);
         write(fileno(stdout), buffer, buffer_len);
       }
@@ -302,11 +305,29 @@ static int juggle(curl_socket_t *sockfdp,
                    (int)sizeof(buffer), buffer_len);
           return FALSE;
         }
-        nread_stdin = read(fileno(stdin), buffer, buffer_len);
-        if(nread_stdin != buffer_len)
-          return FALSE;
-
         logmsg("> %d bytes data, server => client", buffer_len);
+
+        if(buffer_len > (ssize_t)sizeof(buffer)) {
+          logmsg("ERROR: %d bytes of data does not fit within the %d "
+                 "bytes buffer", buffer_len, sizeof(buffer));
+          return FALSE;
+        }
+
+        /*
+         * To properly support huge data chunks, we need to repeat the call
+         * to read() until we're done or it fails.
+         */
+        nread_stdin = 0;
+        do {
+          /* get data in the buffer at the correct position */
+          rc = read(fileno(stdin), &buffer[nread_stdin],
+                    buffer_len - nread_stdin);
+          logmsg("read %d bytes", rc);
+          if(rc <= 0)
+            return FALSE;
+          nread_stdin += rc;
+        } while (nread_stdin < buffer_len);
+
         lograw(buffer, buffer_len);
 
         if(*mode == PASSIVE_LISTEN) {
@@ -353,7 +374,7 @@ static int juggle(curl_socket_t *sockfdp,
          client connecting. */
       sockfd = accept(sockfd, NULL, NULL);
       if(CURL_SOCKET_BAD == sockfd)
-        logmsg("accept() failed\n");
+        logmsg("accept() failed");
       else {
         logmsg("====> Client connect");
         write(fileno(stdout), "CNCT\n", 5);
@@ -378,7 +399,7 @@ static int juggle(curl_socket_t *sockfdp,
       return TRUE;
     }
 
-    sprintf(data, "DATA\n%04x\n", nread_socket);
+    snprintf(data, sizeof(data), "DATA\n%04x\n", nread_socket);
     write(fileno(stdout), data, 10);
     write(fileno(stdout), buffer, nread_socket);
 
@@ -399,10 +420,19 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
 #endif /* ENABLE_IPV6 */
   int flag = 1;
   int rc;
+  int maxretr = 12;
+  int delay= 10;
 
-  if (setsockopt
-      (sock, SOL_SOCKET, SO_REUSEADDR, (void *)&flag,
-       sizeof(flag)) < 0) {
+  rc = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
+       (void *)&flag, sizeof(flag));
+  while ((rc < 0) && maxretr) {
+    maxretr--;
+    go_sleep(delay);
+    delay *= 2; /* double the sleep for next attempt */
+    rc = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
+         (void *)&flag, sizeof(flag));
+  }
+  if (rc < 0) {
     perror("setsockopt(SO_REUSEADDR)");
   }
 
@@ -438,7 +468,7 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
 
     if(getsockname(sock, (struct sockaddr *) &add,
                    &socksize)<0) {
-      fprintf(stderr, "getsockname() failed");
+      logmsg("getsockname() failed with error: %d", SOCKERRNO);
       return CURL_SOCKET_BAD;
     }
     *port = ntohs(add.sin_port);
@@ -447,7 +477,7 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
   /* start accepting connections */
   rc = listen(sock, 4);
   if(0 != rc) {
-    logmsg("listen() failed with error: %d", errno);
+    logmsg("listen() failed with error: %d", SOCKERRNO);
     sclose(sock);
     return CURL_SOCKET_BAD;
   }
@@ -489,6 +519,7 @@ int main(int argc, char *argv[])
   FILE *pidfile;
   char *pidname= (char *)".sockfilt.pid";
   int rc;
+  int error;
   int arg=1;
   enum sockmode mode = PASSIVE_LISTEN; /* default */
 
@@ -570,7 +601,7 @@ int main(int argc, char *argv[])
 
   sock = mksock(use_ipv6);
   if (CURL_SOCKET_BAD == sock) {
-    logmsg("Error opening socket: %d", errno);
+    logmsg("Error opening socket: %d", SOCKERRNO);
     return 1;
   }
 
@@ -631,7 +662,10 @@ int main(int argc, char *argv[])
     logmsg("Wrote pid %d to %s", pid, pidname);
   }
   else {
-    fprintf(stderr, "Couldn't write pid file\n");
+    error = ERRNO;
+    logmsg("fopen() failed with error: %d %s\n", error, strerror(error));
+    logmsg("Error opening file: %s\n", pidname);
+    logmsg("Couldn't write pid file\n");
     sclose(sock);
     return 1;
   }
