@@ -6,7 +6,7 @@
 #                            | (__| |_| |  _ <| |___
 #                             \___|\___/|_| \_\_____|
 #
-# Copyright (C) 1998 - 2005, Daniel Stenberg, <daniel@haxx.se>, et al.
+# Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: testcurl.pl,v 1.29 2005/03/19 01:00:00 bagder Exp $
+# $Id: testcurl.pl,v 1.53 2007-06-28 22:31:18 gknauf Exp $
 ###########################################################################
 
 ###########################
@@ -46,6 +46,7 @@
 # --mktarball=[command]    Command to run after completed test
 # --name=[name]            Set name to report as
 # --nocvsup                Don't update from CVS even though it is a CVS tree
+# --nobuildconf            Don't run buildconf
 # --runtestopts=[options]  Options to pass to runtests.pl
 # --setup=[file name]      File name to read setup from (deprecated)
 # --target=[your os]       Specify your target environment.
@@ -64,10 +65,10 @@ use vars qw($version $fixed $infixed $CURLDIR $CVS $pwd $build $buildlog
             $buildlogname $configurebuild $targetos $confsuffix $binext
             $libext);
 use vars qw($name $email $desc $confopts $runtestopts $setupfile $mktarball
-            $nocvsup $crosscompile);
+            $nocvsup $nobuildconf $crosscompile $timestamp);
 
 # version of this script
-$version='$Revision: 1.29 $';
+$version='$Revision: 1.53 $';
 $fixed=0;
 
 # Determine if we're running from CVS or a canned copy of curl,
@@ -101,6 +102,10 @@ while ($ARGV[0]) {
     $nocvsup=1;
     shift @ARGV;
   }
+  elsif ($ARGV[0] =~ /--nobuildconf/) {
+    $nobuildconf=1;
+    shift @ARGV;
+  }
   elsif ($ARGV[0] =~ /--crosscompile/) {
     $crosscompile=1;
     shift @ARGV;
@@ -124,21 +129,32 @@ if ($^O eq 'MSWin32' || $targetos) {
     # If no target defined on Win32 lets assume vc
     $targetos = 'vc';
   }
-  if ($targetos =~ /vc/ || $targetos =~ /mingw32/ || $targetos =~ /borland/) {
+  if ($targetos =~ /vc/ || $targetos =~ /borland/) {
     $binext = '.exe';
-    $libext = '.lib' if ($targetos =~ /vc/ || $targetos =~ /borland/);
-    $libext = '.a' if ($targetos =~ /mingw32/);
+    $libext = '.lib';
+  }
+  elsif ($targetos =~ /mingw32/) {
+    $binext = '.exe';
+    if ($^O eq 'MSWin32') {
+      $libext = '.a';
+    }
   }
   elsif ($targetos =~ /netware/) {
     $configurebuild = 0;
     $binext = '.nlm';
-    $libext = '.lib';
+    if ($^O eq 'MSWin32') {
+      $libext = '.lib';
+    }
+    else {
+      $libext = '.a';
+    }
   }
 }
 
-if ($^O eq 'MSWin32') {
+if (($^O eq 'MSWin32') &&
+    ($targetos =~ /vc/ || $targetos =~ /mingw32/ || $targetos =~ /borland/)) {
 
-  # Set these things only when building ON Windows, not when simply building
+  # Set these things only when building ON Windows and for Win32 platform.
   # FOR Windows since we might be cross-compiling on another system. Non-
   # Windows builds still default to configure-style builds with no confsuffix.
 
@@ -254,17 +270,27 @@ if ($fixed < 4) {
     close(F);
 }
 
+my $str1066os = 'o' x 1066;
+
+# Set timestamp to the UTC this script is running. Its value might
+# be changed later in the script to the value present in curlver.h
+$timestamp = scalar(gmtime)." UTC";
+
 logit "STARTING HERE"; # first line logged, for scripts to trigger on
+logit 'TRANSFER CONTROL ==== 1120 CHAR LINE' . $str1066os . 'LINE_END';
 logit "NAME = $name";
 logit "EMAIL = $email";
 logit "DESC = $desc";
 logit "CONFOPTS = $confopts";
+logit "CPPFLAGS = ".$ENV{CPPFLAGS};
 logit "CFLAGS = ".$ENV{CFLAGS};
 logit "LDFLAGS = ".$ENV{LDFLAGS};
 logit "CC = ".$ENV{CC};
 logit "target = ".$targetos;
 logit "version = $version"; # script version
-logit "date = ".(scalar gmtime)." UTC";
+logit "date = $timestamp";  # When the test build starts
+
+$str1066os = undef;
 
 # Make $pwd to become the path without newline. We'll use that in order to cut
 # off that path from all possible logs and error messages etc.
@@ -317,12 +343,12 @@ if ($CVS) {
 
   sub cvsup() {
     # update quietly to the latest CVS
-    logit "run cvs up";
     if($nocvsup) {
         logit "Skipping CVS update (--nocvsup)";
         return 1;
     }
     else {
+        logit "run cvs up";
         system("cvs -Q up -dP 2>&1");
     }
 
@@ -347,12 +373,19 @@ if ($CVS) {
   if ($cvsstat != 0) {
     mydie "failed to update from CVS ($cvsstat), exiting";
   }
+  elsif (!$nocvsup) {
+    # Set timestamp to the UTC the CVS update took place.
+    $timestamp = scalar(gmtime)." UTC";
+  }
 
-  # remove possible left-overs from the past
-  unlink "configure";
-  unlink "autom4te.cache";
+  if($nobuildconf) {
+      logit "told to not run buildconf";
+  }
+  elsif ($configurebuild) {
+    # remove possible left-overs from the past
+    unlink "configure";
+    unlink "autom4te.cache";
 
-  if ($configurebuild) {
     # generate the build files
     logit "invoke buildconf, but filter off the silly aclocal warnings";
     open(F, "./buildconf 2>&1 |") or die;
@@ -371,38 +404,66 @@ if ($CVS) {
     else {
       mydie "buildconf was NOT successful";
     }
-
-    if($confopts =~ /--enable-ares/) {
-        logit "run buildconf for ares";
-        chdir "ares";
-        open(F, "./buildconf 2>&1 |") or die;
-        open(LOG, ">$buildlog") or die;
-        while (<F>) {
-            next if /warning: underquoted definition of/;
-            print;
-            print LOG;
-        }
-        close(F);
-        close(LOG);
-        chdir "..";
-    }
-
   }
   else {
     logit "buildconf was successful (dummy message)";
   }
 }
 
+# Set timestamp to the one in curlver.h if this isn't a CVS test build.
+if ((-f "include/curl/curlver.h") &&
+    (open(F, "<include/curl/curlver.h"))) {
+  while (<F>) {
+    chomp;
+    if ($_ =~ /^\#define\s+LIBCURL_TIMESTAMP\s+\"(.+)\".*$/) {
+      my $stampstring = $1;
+      if ($stampstring !~ /CVS/) {
+          $stampstring =~ s/\s+UTC//;
+          $timestamp = $stampstring." UTC";
+      }
+      last;
+    }
+  }
+  close(F);
+}
+
+# Show timestamp we are using for this test build.
+logit "timestamp = $timestamp";
+
 if ($configurebuild) {
   if (-f "configure") {
-    logit "configure created";
+    logit "configure created (at least it exists)";
   } else {
-    mydie "no configure created";
+    mydie "no configure created/found";
   }
 } else {
   logit "configure created (dummy message)"; # dummy message to feign success
 }
 
+sub findinpath {
+  my $c;
+  my $e;
+  my $x='';
+  $x='.exe' if ($^O eq 'MSWin32');
+  my $s=':';
+  $s=';' if ($^O eq 'MSWin32');
+  my $p=$ENV{'PATH'};
+  my @pa = split($s, $p);
+  for $c (@_) {
+    for $e (@pa) {
+      if( -x "$e/$c$x") {
+        return $c;
+      }
+    }
+  }
+}
+
+my $make = findinpath("gmake", "make", "nmake");
+if(!$make) {
+    mydie "Couldn't find make in the PATH";
+}
+logit "going with $make as make";
+
 # change to build dir
 chdir "$pwd/$build";
 
@@ -416,6 +477,7 @@ if ($configurebuild) {
     mydie "configure didn't work";
   }
 } else {
+  logit "copying files to build dir ...";
   if (($^O eq 'MSWin32') && ($targetos !~ /netware/)) {
     system("xcopy /s /q ..\\$CURLDIR .");
     system("buildconf.bat");
@@ -423,8 +485,8 @@ if ($configurebuild) {
   elsif (($^O eq 'linux') || ($targetos =~ /netware/)) {
     system("cp -afr ../$CURLDIR/* ."); 
     system("cp -af ../$CURLDIR/Makefile.dist Makefile"); 
-    system("make -i -C lib -f Makefile.$targetos prebuild");
-    system("make -i -C src -f Makefile.$targetos prebuild");
+    system("$make -i -C lib -f Makefile.$targetos prebuild");
+    system("$make -i -C src -f Makefile.$targetos prebuild");
   }
 }
 
@@ -435,26 +497,27 @@ while (<F>) {
 }
 close(F);
 
-logit "display src/config$confsuffix.h";
-open(F, "src/config$confsuffix.h") or die "src/config$confsuffix.h: $!";
-while (<F>) {
-  print if /^ *#/;
-}
-close(F);
-
 if (grepfile("define USE_ARES", "lib/config$confsuffix.h")) {
   logit "setup to build ares";
 
+  logit "display ares/config$confsuffix.h";
+  if(open(F, "ares/config$confsuffix.h")) {
+      while (<F>) {
+          print if /^ *#/;
+      }
+      close(F);
+  }
+
   logit "build ares";
   chdir "ares";
 
   if ($targetos && !$configurebuild) {
-      logit "make -f Makefile.$targetos";
-      open(F, "make -f Makefile.$targetos 2>&1 |") or die;
+      logit "$make -f Makefile.$targetos";
+      open(F, "$make -f Makefile.$targetos 2>&1 |") or die;
   }
   else {
-      logit "make";
-      open(F, "make 2>&1 |") or die;
+      logit "$make";
+      open(F, "$make 2>&1 |") or die;
   }
   while (<F>) {
     s/$pwd//g;
@@ -473,8 +536,8 @@ if (grepfile("define USE_ARES", "lib/config$confsuffix.h")) {
 }
 
 if ($configurebuild) {
-  logit "make -i";
-  open(F, "make -i 2>&1 |") or die;
+  logit "$make -i";
+  open(F, "$make -i 2>&1 |") or die;
   while (<F>) {
     s/$pwd//g;
     print;
@@ -482,17 +545,17 @@ if ($configurebuild) {
   close(F);
 }
 else {
-  logit "make -i $targetos";
+  logit "$make -i $targetos";
   if ($^O eq 'MSWin32') {
     if ($targetos =~ /vc/) {
       open(F, "nmake -i $targetos|") or die;
     }
     else {
-      open(F, "make -i $targetos |") or die;
+      open(F, "$make -i $targetos |") or die;
     }
   }
   else {
-    open(F, "make -i $targetos 2>&1 |") or die;
+    open(F, "$make -i $targetos 2>&1 |") or die;
   }
   while (<F>) {
     s/$pwd//g;
@@ -502,17 +565,17 @@ else {
 }
 
 if (-f "lib/libcurl$libext") {
-  logit "lib/libcurl was created fine (libcurl$libext)";
+  logit "libcurl was created fine (libcurl$libext)";
 }
 else {
-  logit "lib/libcurl was not created (libcurl$libext)";
+  mydie "libcurl was not created (libcurl$libext)";
 }
 
 if (-f "src/curl$binext") {
-  logit "src/curl was created fine (curl$binext)";
+  logit "curl was created fine (curl$binext)";
 }
 else {
-  mydie "src/curl was not created (curl$binext)";
+  mydie "curl was not created (curl$binext)";
 }
 
 if ($targetos =~ /netware/) {
@@ -522,7 +585,12 @@ if ($targetos =~ /netware/) {
 }
 elsif(!$crosscompile) {
   logit "display curl$binext --version output";
-  system("./src/curl$binext --version");
+  open(F, "./src/curl$binext --version|");
+  while(<F>) {
+      print;
+      print LOG;
+  }
+  close(F);
 }
 
 if ($configurebuild && !$crosscompile) {
@@ -530,8 +598,8 @@ if ($configurebuild && !$crosscompile) {
   if($runtestopts) {
       $o = "TEST_F=\"$runtestopts\" ";
   }
-  logit "make ${o}test-full";
-  open(F, "make ${o}test-full 2>&1 |") or die;
+  logit "$make -k ${o}test-full";
+  open(F, "$make -k ${o}test-full 2>&1 |") or die;
   open(LOG, ">$buildlog") or die;
   while (<F>) {
     s/$pwd//g;
